function [x]  =  specsubt(y,N,beta,gain);
% SPECSUBT.M
% ========== 
%
% Spectral Filter for speech enhancement
%
% Filters a noisy speech vector y, to give enhanced speech vector x.
%
% x  =  specsubt(y, N, beta, gain);
%
% y 	 = noisy input vector (user prompted if no input provided -- Windows PC only)
% N 	 = analysis frame length (default = 160)
% beta = smoothing parameter for the signal estimate (default = 0.97)
% gain = gain function, where ....
%		gain = 'SS' => spectral subtraction
%		gain = 'WF' => Weiner Filter
%		gain = 'MM' => McAulay and Malpass
%		gain = 'EM' => Ephraim and Malah (default)
%
% Hints for use on a Windows PC
% =============================
% Use [y,fs] = wavread('FileName.wav'); to read data from a Windows
% wave file into the vector y.
%
% Use soundsc(x,fs) to play the resulting enhanced speech vector.
% Use wavwrite(x,fs,'OutputFileName.wav'); to write enhanced speech to a file
%
% Written by Dr. Graham Eatwell, Adaptive Audio Inc., 2001


% set default inputs
if nargin<1
	[FileName, PathName] = uigetfile( '*.wav', 'Select wave file for noise speech' );
	[y,fs] = wavread([PathName FileName]); 		% read data from a Windows
end

if nargin<2
   N = 160;						% analysis frame length
end

if nargin<3
   beta = 0.97;				% recursive smoothing parameter for speech estimate
end

if nargin<4  | length(gain)~=2
   gain = 'EM';				% Ephraim and Malah gain function
else
   gain = upper(gain);
end

m = length(y)/(N/2)-1;	% total number of frames to process
x = zeros(size(y));		% output storage
h = [0;hanning(N-1)];		% Hanning window

% parameters
kfft = 1 + N/2;		% number of frequency bins to process
alpha = 0.0;                % update rate for noise estimate
averages = 20;             % number of frames for initial noise estimate

% variables
F = zeros(kfft,1);		% filter coefficients
sig2 = zeros(kfft,1);		% aprior signal estimate

% set some minimum values to avoid division by zero
ms = mean(y.^2);
min_power  = ms/1000;
min_noise  = ms/1000;
min_signal = ms/1000;

% initialize noise estimate (assumes that first frames are noise only)
frame = 1:N;					% frame index
noise = zeros(kfft,1);	
for n=1:averages
   Y = fft(y(frame).*h);	% FFT of windowed data
   Y = Y(1:kfft);
   noise = noise + Y.*conj(Y);
   frame = frame + N/2;		% advance to next frame
end
noise = noise/averages;		% get average noise power
noise = max(min_noise,noise);	

% main processing loop
frame = 1:N;					% initial frame index
for n = 1:m
   % ANALYSIS
	Y = fft(y(frame).*h);   % FFT of windowed data
   Y = Y(1:kfft);				% other frequency bins calculated from symmetry
   power = max(min_power,Y.*conj(Y));

	% FILTER CALCUATION
	% signal estimate
   sig1 = max(0,power-noise);
	signal = (1-beta)*sig1 + beta*sig2;
   
   % noise estimate (only done if there is no speech in the current frame)
   % Voice activity detect (threshold detector)
   sumN = sum(noise);
   sumP = sum(power);
   T = 0.3;												% normalized threshold
   if (sumP-sumN) < T*sumN;						% find pauses
		noise = noise + alpha*(power-noise);	% recursive smoother
      noise = max(noise,min_noise);				% clip below
   end   
   
   % coefficient calculation
   if gain=='SS'										% spectral subtraction
      F = sqrt(max(power-noise,0)./power); 
   elseif gain == 'WF'								% Weiner filter
      F = max(power-noise,0)./power; 
   elseif gain == 'MM'								% McAulay and Malpass
      g = max(power-noise,0)./power;
      epsi = 0.5;
      e = exp(-epsi);
      b = e*besseli(0,2*sqrt(epsi./(1-g)));
      F = 0.5*(1+sqrt(g))*e/(1+e).*b./(1+b);
   elseif gain == 'EM'								% Ephraim and Malah
      g = power./noise;
      epsi = signal./noise;
      v = epsi./(1+epsi).*g;
      F = gamma(1.5)*sqrt(v)./g.*exp(-v/2).*((1+v).*besseli(0,v/2)+v.*besseli(1,v/2));
   end
   
   % clip gain at unity
   F = min(F,1);
   
   % FILTERING
	X = F.*Y;
   sig2 = X.*conj(X);      
   
   % SYNTHESIS (overlap--add)
  	XX = [X; (conj(X((N/2):-1:2)))];			   % set rest of spectrum by symmetry
	x(frame) = x(frame) + real(ifft(XX));	   % inverse Fourier transform
   
   % FRAME ADVANCE
   frame = frame + N/2;			
end;

% zero pad output to be same length as input
L = length(y);
if length(x)<L
   x(length(x)+1:L) = zeros(size( x(1:L-length(x)) ));
end;
